@@ -4,38 +4,51 @@ from esphome import automation
 from esphome.automation import maybe_simple_id
 from esphome.components import mqtt
 from esphome.const import (
-    CONF_DISABLED_BY_DEFAULT,
     CONF_ID,
-    CONF_INTERNAL,
     CONF_MQTT_ID,
     CONF_OSCILLATING,
     CONF_OSCILLATION_COMMAND_TOPIC,
     CONF_OSCILLATION_STATE_TOPIC,
     CONF_SPEED,
+    CONF_SPEED_LEVEL_COMMAND_TOPIC,
+    CONF_SPEED_LEVEL_STATE_TOPIC,
     CONF_SPEED_COMMAND_TOPIC,
     CONF_SPEED_STATE_TOPIC,
-    CONF_NAME,
+    CONF_ON_SPEED_SET,
     CONF_ON_TURN_OFF,
     CONF_ON_TURN_ON,
     CONF_TRIGGER_ID,
+    CONF_DIRECTION,
 )
 from esphome.core import CORE, coroutine_with_priority
+from esphome.cpp_helpers import setup_entity
 
 IS_PLATFORM_COMPONENT = True
 
 fan_ns = cg.esphome_ns.namespace("fan")
-FanState = fan_ns.class_("FanState", cg.Nameable, cg.Component)
+FanState = fan_ns.class_("FanState", cg.EntityBase, cg.Component)
 MakeFan = cg.Application.struct("MakeFan")
 
+FanDirection = fan_ns.enum("FanDirection")
+FAN_DIRECTION_ENUM = {
+    "FORWARD": FanDirection.FAN_DIRECTION_FORWARD,
+    "REVERSE": FanDirection.FAN_DIRECTION_REVERSE,
+}
+
 # Actions
 TurnOnAction = fan_ns.class_("TurnOnAction", automation.Action)
 TurnOffAction = fan_ns.class_("TurnOffAction", automation.Action)
 ToggleAction = fan_ns.class_("ToggleAction", automation.Action)
+CycleSpeedAction = fan_ns.class_("CycleSpeedAction", automation.Action)
 
 FanTurnOnTrigger = fan_ns.class_("FanTurnOnTrigger", automation.Trigger.template())
 FanTurnOffTrigger = fan_ns.class_("FanTurnOffTrigger", automation.Trigger.template())
+FanSpeedSetTrigger = fan_ns.class_("FanSpeedSetTrigger", automation.Trigger.template())
 
-FAN_SCHEMA = cv.NAMEABLE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
+FanIsOnCondition = fan_ns.class_("FanIsOnCondition", automation.Condition.template())
+FanIsOffCondition = fan_ns.class_("FanIsOffCondition", automation.Condition.template())
+
+FAN_SCHEMA = cv.ENTITY_BASE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
     {
         cv.GenerateID(): cv.declare_id(FanState),
         cv.OnlyWith(CONF_MQTT_ID, "mqtt"): cv.declare_id(mqtt.MQTTFanComponent),
@@ -45,6 +58,12 @@ FAN_SCHEMA = cv.NAMEABLE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
         cv.Optional(CONF_OSCILLATION_COMMAND_TOPIC): cv.All(
             cv.requires_component("mqtt"), cv.subscribe_topic
         ),
+        cv.Optional(CONF_SPEED_LEVEL_STATE_TOPIC): cv.All(
+            cv.requires_component("mqtt"), cv.publish_topic
+        ),
+        cv.Optional(CONF_SPEED_LEVEL_COMMAND_TOPIC): cv.All(
+            cv.requires_component("mqtt"), cv.subscribe_topic
+        ),
         cv.Optional(CONF_SPEED_STATE_TOPIC): cv.All(
             cv.requires_component("mqtt"), cv.publish_topic
         ),
@@ -61,15 +80,17 @@ FAN_SCHEMA = cv.NAMEABLE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
                 cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(FanTurnOffTrigger),
             }
         ),
+        cv.Optional(CONF_ON_SPEED_SET): automation.validate_automation(
+            {
+                cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(FanSpeedSetTrigger),
+            }
+        ),
     }
 )
 
 
 async def setup_fan_core_(var, config):
-    cg.add(var.set_name(config[CONF_NAME]))
-    cg.add(var.set_disabled_by_default(config[CONF_DISABLED_BY_DEFAULT]))
-    if CONF_INTERNAL in config:
-        cg.add(var.set_internal(config[CONF_INTERNAL]))
+    await setup_entity(var, config)
 
     if CONF_MQTT_ID in config:
         mqtt_ = cg.new_Pvariable(config[CONF_MQTT_ID], var)
@@ -87,6 +108,18 @@ async def setup_fan_core_(var, config):
                     config[CONF_OSCILLATION_COMMAND_TOPIC]
                 )
             )
+        if CONF_SPEED_LEVEL_STATE_TOPIC in config:
+            cg.add(
+                mqtt_.set_custom_speed_level_state_topic(
+                    config[CONF_SPEED_LEVEL_STATE_TOPIC]
+                )
+            )
+        if CONF_SPEED_LEVEL_COMMAND_TOPIC in config:
+            cg.add(
+                mqtt_.set_custom_speed_level_command_topic(
+                    config[CONF_SPEED_LEVEL_COMMAND_TOPIC]
+                )
+            )
         if CONF_SPEED_STATE_TOPIC in config:
             cg.add(mqtt_.set_custom_speed_state_topic(config[CONF_SPEED_STATE_TOPIC]))
         if CONF_SPEED_COMMAND_TOPIC in config:
@@ -100,6 +133,9 @@ async def setup_fan_core_(var, config):
     for conf in config.get(CONF_ON_TURN_OFF, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
         await automation.build_automation(trigger, [], conf)
+    for conf in config.get(CONF_ON_SPEED_SET, []):
+        trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
+        await automation.build_automation(trigger, [], conf)
 
 
 async def register_fan(var, config):
@@ -143,6 +179,9 @@ async def fan_turn_off_to_code(config, action_id, template_arg, args):
             cv.Required(CONF_ID): cv.use_id(FanState),
             cv.Optional(CONF_OSCILLATING): cv.templatable(cv.boolean),
             cv.Optional(CONF_SPEED): cv.templatable(cv.int_range(1)),
+            cv.Optional(CONF_DIRECTION): cv.templatable(
+                cv.enum(FAN_DIRECTION_ENUM, upper=True)
+            ),
         }
     ),
 )
@@ -155,9 +194,41 @@ async def fan_turn_on_to_code(config, action_id, template_arg, args):
     if CONF_SPEED in config:
         template_ = await cg.templatable(config[CONF_SPEED], args, int)
         cg.add(var.set_speed(template_))
+    if CONF_DIRECTION in config:
+        template_ = await cg.templatable(config[CONF_DIRECTION], args, FanDirection)
+        cg.add(var.set_direction(template_))
     return var
 
 
+@automation.register_action("fan.cycle_speed", CycleSpeedAction, FAN_ACTION_SCHEMA)
+async def fan_cycle_speed_to_code(config, action_id, template_arg, args):
+    paren = await cg.get_variable(config[CONF_ID])
+    return cg.new_Pvariable(action_id, template_arg, paren)
+
+
+@automation.register_condition(
+    "fan.is_on",
+    FanIsOnCondition,
+    automation.maybe_simple_id(
+        {
+            cv.Required(CONF_ID): cv.use_id(FanState),
+        }
+    ),
+)
+@automation.register_condition(
+    "fan.is_off",
+    FanIsOffCondition,
+    automation.maybe_simple_id(
+        {
+            cv.Required(CONF_ID): cv.use_id(FanState),
+        }
+    ),
+)
+async def fan_is_on_off_to_code(config, condition_id, template_arg, args):
+    paren = await cg.get_variable(config[CONF_ID])
+    return cg.new_Pvariable(condition_id, template_arg, paren)
+
+
 @coroutine_with_priority(100.0)
 async def to_code(config):
     cg.add_define("USE_FAN")
