@@ -9,9 +9,11 @@ namespace max31855 {
 
 class MAX31855Sensor : public sensor::Sensor,
                        public PollingComponent,
-                       public spi::SPIDevice<spi::BIT_ORDER_MSB_FIRST, spi::CLOCK_POLARITY_LOW,
-                                             spi::CLOCK_PHASE_LEADING, spi::DATA_RATE_4MHZ> {
+                       public spi::SPIDevice<spi::BIT_ORDER_MSB_FIRST, spi::CLOCK_POLARITY_HIGH,
+                                             spi::CLOCK_PHASE_TRAILING, spi::DATA_RATE_4MHZ> {
  public:
+  void set_reference_sensor(sensor::Sensor *temperature_sensor) { temperature_reference_ = temperature_sensor; }
+
   void setup() override;
   void dump_config() override;
   float get_setup_priority() const override;
@@ -20,6 +22,7 @@ class MAX31855Sensor : public sensor::Sensor,
 
  protected:
   void read_data_();
+  sensor::Sensor *temperature_reference_{nullptr};
 };
 
 }  // namespace max31855
