@@ -2,7 +2,9 @@ from string import ascii_letters, digits
 import esphome.config_validation as cv
 import esphome.codegen as cg
 from esphome.components import color
-
+from esphome.const import (
+    CONF_VISIBLE,
+)
 from . import CONF_NEXTION_ID
 from . import Nextion
 
@@ -25,20 +27,17 @@ CONF_BACKGROUND_PRESSED_COLOR = "background_pressed_color"
 CONF_FOREGROUND_COLOR = "foreground_color"
 CONF_FOREGROUND_PRESSED_COLOR = "foreground_pressed_color"
 CONF_FONT_ID = "font_id"
-CONF_VISIBLE = "visible"
 
 
 def NextionName(value):
-    valid_chars = ascii_letters + digits + "."
+    valid_chars = f"{ascii_letters + digits}."
     if not isinstance(value, str) or len(value) > 29:
         raise cv.Invalid("Must be a string less than 29 characters")
 
     for char in value:
         if char not in valid_chars:
             raise cv.Invalid(
-                "Must only consist of upper/lowercase characters, numbers and the period '.'. The character '{}' cannot be used.".format(
-                    char
-                )
+                f"Must only consist of upper/lowercase characters, numbers and the period '.'. The character '{char}' cannot be used."
             )
 
     return value
