@@ -19,7 +19,7 @@ CONFIG_SCHEMA = cv.All(
 )
 
 
-def to_code(config):
+async def to_code(config):
     var = cg.new_Pvariable(config[CONF_ID])
-    yield pn532.setup_pn532(var, config)
-    yield i2c.register_i2c_device(var, config)
+    await pn532.setup_pn532(var, config)
+    await i2c.register_i2c_device(var, config)
