@@ -1,5 +1,6 @@
 import esphome.codegen as cg
 import esphome.config_validation as cv
+import esphome.final_validate as fv
 from esphome import pins
 from esphome.const import (
     CONF_CLK_PIN,
@@ -9,7 +10,7 @@ from esphome.const import (
     CONF_SPI_ID,
     CONF_CS_PIN,
 )
-from esphome.core import coroutine, coroutine_with_priority
+from esphome.core import coroutine_with_priority, CORE
 
 CODEOWNERS = ["@esphome/core"]
 spi_ns = cg.esphome_ns.namespace("spi")
@@ -31,20 +32,23 @@ CONFIG_SCHEMA = cv.All(
 
 
 @coroutine_with_priority(1.0)
-def to_code(config):
+async def to_code(config):
     cg.add_global(spi_ns.using)
     var = cg.new_Pvariable(config[CONF_ID])
-    yield cg.register_component(var, config)
+    await cg.register_component(var, config)
 
-    clk = yield cg.gpio_pin_expression(config[CONF_CLK_PIN])
+    clk = await cg.gpio_pin_expression(config[CONF_CLK_PIN])
     cg.add(var.set_clk(clk))
     if CONF_MISO_PIN in config:
-        miso = yield cg.gpio_pin_expression(config[CONF_MISO_PIN])
+        miso = await cg.gpio_pin_expression(config[CONF_MISO_PIN])
         cg.add(var.set_miso(miso))
     if CONF_MOSI_PIN in config:
-        mosi = yield cg.gpio_pin_expression(config[CONF_MOSI_PIN])
+        mosi = await cg.gpio_pin_expression(config[CONF_MOSI_PIN])
         cg.add(var.set_mosi(mosi))
 
+    if CORE.is_esp32:
+        cg.add_library("SPI", None)
+
 
 def spi_device_schema(cs_pin_required=True):
     """Create a schema for an SPI device.
@@ -61,10 +65,32 @@ def spi_device_schema(cs_pin_required=True):
     return cv.Schema(schema)
 
 
-@coroutine
-def register_spi_device(var, config):
-    parent = yield cg.get_variable(config[CONF_SPI_ID])
+async def register_spi_device(var, config):
+    parent = await cg.get_variable(config[CONF_SPI_ID])
     cg.add(var.set_spi_parent(parent))
     if CONF_CS_PIN in config:
-        pin = yield cg.gpio_pin_expression(config[CONF_CS_PIN])
+        pin = await cg.gpio_pin_expression(config[CONF_CS_PIN])
         cg.add(var.set_cs_pin(pin))
+
+
+def final_validate_device_schema(name: str, *, require_mosi: bool, require_miso: bool):
+    hub_schema = {}
+    if require_miso:
+        hub_schema[
+            cv.Required(
+                CONF_MISO_PIN,
+                msg=f"Component {name} requires this spi bus to declare a miso_pin",
+            )
+        ] = cv.valid
+    if require_mosi:
+        hub_schema[
+            cv.Required(
+                CONF_MOSI_PIN,
+                msg=f"Component {name} requires this spi bus to declare a mosi_pin",
+            )
+        ] = cv.valid
+
+    return cv.Schema(
+        {cv.Required(CONF_SPI_ID): fv.id_declaration_match_schema(hub_schema)},
+        extra=cv.ALLOW_EXTRA,
+    )
